package CardGames.cards;

/**
 * This class implements a card, the representation of a physical card with a value and a suit.
 * No assumptions of deck size are made.
 *
 * @author Mark Fitzgerald
 * @version 2002.Jan.04
 */
public class SuitedCard implements Comparable
{
    /**
     * Default constructor.  Sets the value and suit to 0.
     */
    public SuitedCard()
    {
        mValue = 0;
        mSuit = 0;
    }

    /**
     * The constructor for cards.
     *
     * @param value The value of the card.  By convention, between 1 and 13, inclusive.  However, all integer values are valid.
     * @param suit The suit of the card.  By convention, between 1 and 4, inclusive.  However, all integer values are valid.
     */
    public SuitedCard(int value, int suit)
    {
        mValue = value;
        mSuit = suit;
    }

    /**
     * Return whether the argument card has the same value and suit as this card.
     * @param otherCard The card to compare this card to in value and suit.
     * @return True if the same card, false otherwise.
     */
    public boolean equals(Object otherCard)
    {
        if ((((SuitedCard)otherCard).mValue == mValue) && (((SuitedCard)otherCard).mSuit == mSuit))
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * Compare this card with one other, returning based on the card's value only.
     * @return -1, 0 or 1, as per the Comparable interface specification.
     */
    public int compareTo(Object other)
    {
        int otherValue = ((SuitedCard)other).mValue;
        if (mValue < otherValue)
        {
            return -1;
        }
        else if (mValue == otherValue)
        {
            return 0;
        }
        else
        {
            return 1;
        }
    }

    /**
     * Return whether the argument card has the same suit as this card.
     * @param otherCard The card to compare this card's suit to.
     * @return True if the same suit, false otherwise.
     */
    public boolean sameSuit(SuitedCard otherCard)
    {
        return (mSuit == otherCard.mSuit);
    }
    
    /**
     * Return whether the argument card has the same value as this card.
     * @param otherCard The card to compare this card's value to.
     * @return True if the same value, false otherwise.
     */
    public boolean sameValue(SuitedCard otherCard)
    {
        return (mValue == otherCard.mValue);
    }

    /**
     * @return A string representation of the card.  The returned string has the form "xx_yy", where xx is the
     * value, and yy is the suit.
     */
    public String toString()
    {
        return new String(mValue + "_" + mSuit);
    }

    /** The value of the card. */
    public int mValue=0;
    /** The suit of the card. */
    public int mSuit=0;
}