package CardGames.cribbage;

/** A container for the in-hand scoring value of a cribbage hand. */
public class CribbageHandCount
{
    /** Default constructor - set all points values to 0. */
    public CribbageHandCount() {}
    
    /** Constructor - set all point values to the handed-in values.
     * @throws IllegalArgumentException if any of the point values is negative. */
    public CribbageHandCount(int iFifteensPoints, int iPairsPoints, int iRunsPoints, int iFlushPoints, int iHisNobsPoints)
    {
        if (iFifteensPoints < 0 || iPairsPoints < 0 || iRunsPoints < 0 || iFlushPoints < 0 || iHisNobsPoints < 0)
        {
            throw new IllegalArgumentException();
        }
        this.iFifteensPoints = iFifteensPoints;
        this.iPairsPoints = iPairsPoints;
        this.iRunsPoints = iRunsPoints;
        this.iFlushPoints = iFlushPoints;
        this.iHisNobsPoints = iHisNobsPoints;
    }

    /** Adds the handed-in count to this count, and saves the result in this count. */
    public void Add(CribbageHandCount oCount)
    {
        iFifteensPoints += oCount.iFifteensPoints;
        iPairsPoints += oCount.iPairsPoints;
        iRunsPoints += oCount.iRunsPoints;
        iFlushPoints += oCount.iFlushPoints;
        iHisNobsPoints += oCount.iHisNobsPoints;
    }
    
    public int GetPoints() { return iFifteensPoints+iPairsPoints+iRunsPoints+iFlushPoints+iHisNobsPoints; }
    
    /** @return <code>true</code> if all 5 point categories are equal in this and the
     * other object, <code>false</code> otherwise. */
    public boolean equals(Object o)
    {
        CribbageHandCount oCount = (CribbageHandCount)o;
        return (iFifteensPoints == oCount.iFifteensPoints && iPairsPoints == oCount.iPairsPoints
            && iRunsPoints == oCount.iRunsPoints && iFlushPoints == oCount.iFlushPoints
            && iHisNobsPoints == oCount.iHisNobsPoints);
    }
    
    public int GetFifteensPoints() { return iFifteensPoints; }
    public int GetPairsPoints() { return iPairsPoints; }
    public int GetRunsPoints() { return iRunsPoints; }
    public int GetFlushPoints() { return iFlushPoints; }
    public int GetHisNobsPoints() { return iHisNobsPoints; }
    
    private int iFifteensPoints = 0, iPairsPoints = 0, iRunsPoints = 0, iFlushPoints = 0, iHisNobsPoints = 0;
}