package CardGames.cribbage;

import java.util.Vector;

public class CribbageHandCountCounts
{
    /** Create an empty array of counts. */
    public CribbageHandCountCounts() {}
    
    /** Add the given count to our count of counts.
     * @throws IllegalArgumentException if any of the points exceed the maxima for a standard 4 card plus one turn-up hand. */
    public void Add(CribbageHandCount oCount)
    {
        int iFifteensPoints = oCount.GetFifteensPoints();
        int iPairsPoints = oCount.GetPairsPoints();
        int iRunsPoints = oCount.GetRunsPoints();
        int iFlushPoints = oCount.GetFlushPoints();
        int iHisNobsPoints = oCount.GetHisNobsPoints();
        if (iFifteensPoints > s_iMaxFifteenPoints || iPairsPoints > s_iMaxPairsPoints || iRunsPoints > s_iMaxRunsPoints
            || iFlushPoints > s_iMaxFlushPoints || iHisNobsPoints > s_iMaxHisNobsPoints)
        {
            throw new IllegalArgumentException("Bad points: 15=" + iFifteensPoints + ", pair=" + iPairsPoints
                + ", runs=" + iRunsPoints + ", flush=" + iFlushPoints + ", nobs=" + iHisNobsPoints + ".");
        }
        
        ++m_alFifteenCountCounts[iFifteensPoints];
        ++m_alPairCountCounts[iPairsPoints];
        ++m_alRunCountCounts[iRunsPoints];
        ++m_alFlushCountCounts[iFlushPoints];
        ++m_alHisNobsCountCounts[iHisNobsPoints];
    }
    
    /** Add the given count of counts to our count of counts. */
    public void Add(CribbageHandCountCounts oCounts)
    {
        AddToArray(m_alFifteenCountCounts, oCounts.m_alFifteenCountCounts);
        AddToArray(m_alPairCountCounts, oCounts.m_alPairCountCounts);
        AddToArray(m_alRunCountCounts, oCounts.m_alRunCountCounts);
        AddToArray(m_alFlushCountCounts, oCounts.m_alFlushCountCounts);
        AddToArray(m_alHisNobsCountCounts, oCounts.m_alHisNobsCountCounts);
    }
    private void AddToArray(long[] alAddToHere, long[] alAddFromHere)
    {
        assert alAddToHere.length == alAddFromHere.length : "Coding error.";
        for (int iIndex=alAddToHere.length-1; iIndex>=0; --iIndex)
        {
            alAddToHere[iIndex] += alAddFromHere[iIndex];
        }
    }
    
    /** Return the total points in all hands in this array. */
    public long GetPoints()
    {
        long lTotalPoints = 0L;
        lTotalPoints += SumPointsArray(m_alFifteenCountCounts);
        lTotalPoints += SumPointsArray(m_alPairCountCounts);
        lTotalPoints += SumPointsArray(m_alRunCountCounts);
        lTotalPoints += SumPointsArray(m_alFlushCountCounts);
        lTotalPoints += SumPointsArray(m_alHisNobsCountCounts);
        return lTotalPoints;
    }
    
    /** @return The average points scored across all hands in this object. */
    public double GetAveragePoints()
    {
        return ((double)GetPoints()) / ((double)GetNumberOfCounts());
    }
    
    /** @return The number of hand counts in this object. */
    public long GetNumberOfCounts()
    {
        assert s_iMaxHisNobsPoints==1;
        return m_alHisNobsCountCounts[0] + m_alHisNobsCountCounts[1];
    }
    
    public double GetAverageFifteensPoints()
    {
        return ((double)SumPointsArray(m_alFifteenCountCounts)) / ((double)GetNumberOfCounts());
    }
    public double GetAveragePairsPoints()
    {
        return ((double)SumPointsArray(m_alPairCountCounts)) / ((double)GetNumberOfCounts());
    }
    public double GetAverageRunsPoints()
    {
        return ((double)SumPointsArray(m_alRunCountCounts)) / ((double)GetNumberOfCounts());
    }
    public double GetAverageFlushPoints()
    {
        return ((double)SumPointsArray(m_alFlushCountCounts)) / ((double)GetNumberOfCounts());
    }
    public double GetAverageHisNobsPoints()
    {
        return ((double)SumPointsArray(m_alHisNobsCountCounts)) / ((double)GetNumberOfCounts());
    }
    
    /** Sum an array of count counts and return the total number of points.  The array is assumed to contain the number
     * of times a N-point hand was seen in array index N.
     * @param lCountCounts The count counts.
     * @return The total points represented by the count counts. */
    private long SumPointsArray(long[] lCountCounts)
    {
        long lTotalPoints = 0L;
        for (int iCount=lCountCounts.length-1; iCount>0; --iCount)
        {
            lTotalPoints += lCountCounts[iCount] * iCount;
        }
        return lTotalPoints;
    }
    
    private static final int s_iMaxFifteenPoints = 16, s_iMaxPairsPoints = 12, s_iMaxRunsPoints = 12,
        s_iMaxFlushPoints = 5, s_iMaxHisNobsPoints = 1;
    private long[] m_alFifteenCountCounts = new long[s_iMaxFifteenPoints+1];
    private long[] m_alPairCountCounts = new long[s_iMaxPairsPoints+1];
    private long[] m_alRunCountCounts = new long[s_iMaxRunsPoints+1];
    private long[] m_alFlushCountCounts = new long[s_iMaxFlushPoints+1];
    private long[] m_alHisNobsCountCounts = new long[s_iMaxHisNobsPoints+1];
}