package CardGames.cribbage;

import CardGames.cards.StandardCard;
import java.util.*;
import junit.framework.*;
import org.netbeans.junit.*;

public class UtilitiesTest extends NbTestCase
{
    public UtilitiesTest(java.lang.String testName)
    {
        super(testName);
    }
    
    public static void main(java.lang.String[] args)
    {
        junit.textui.TestRunner.run(suite());
    }
    
    public static Test suite()
    {
        TestSuite suite = new NbTestSuite(UtilitiesTest.class);
        
        return suite;
    }
    
    /** Test of countingValue method, of class CardGames.cribbage.Utilities. */
    public void testCountingValue()
    {
        //Create some cards, and check their counting values.
        int[] aiCounts = new int[] { 1, 5, 10, 10, 10, 7 };
        try {
            for (int iCount = 0; iCount < aiCounts.length; ++iCount)
            {
                assertEquals(aiCounts[iCount], Utilities.countingValue(new StandardCard(astrCards[iCount])));
            }
        } catch (StandardCard.InvalidCard e) { fail(); }
    }
    
    /** Test of getCacheStatistics method, of class CardGames.cribbage.Utilities. */
    public void testGetCacheStatisticsAndCountHand()
    {
        try {
            verifyCacheStats(0, 0);
            //Create a hand with ac, 5d, qh, ks, with 10c turnup.
            StandardCard[] aobjHand = new StandardCard[4];
            StandardCard objTurnup = new StandardCard(astrCards[4]);
            for (int iCard = 0; iCard < aobjHand.length; ++iCard)
            {
                aobjHand[iCard] = new StandardCard(astrCards[iCard]);
            }
            verifyCounts(aobjHand, objTurnup, false, new CribbageHandCount(6, 0, 0, 0, 0));
            verifyCacheStats(1, 0);
            
            //Modify the hand a little, to ac, 5d, 7d, 6h, with 10c turnup.
            aobjHand[2] = new StandardCard(astrCards[5]);
            aobjHand[3] = new StandardCard(astrCards[6]);
            verifyCounts(aobjHand, objTurnup, false, new CribbageHandCount(2, 0, 3, 0, 0));
            verifyCacheStats(2, 0);
            verifyCounts(aobjHand, objTurnup, false, new CribbageHandCount(2, 0, 3, 0, 0));
            verifyCacheStats(2, 1);
        } catch (StandardCard.InvalidCard e) { fail(); }
    }
    private void verifyCacheStats(int iPuts, int iGets)
    {
        assertEquals(iPuts, Utilities.getCacheStatistics()[0]);
        assertEquals(iGets, Utilities.getCacheStatistics()[1]);
    }
    private void verifyCounts(StandardCard[] aobjHand, StandardCard aobjTurnup,
        boolean bIsCrib, CribbageHandCount objExpectedCount)
    {
        CribbageHandCount objCount = Utilities.countHand(aobjHand, aobjTurnup, bIsCrib);
        assertEquals(objExpectedCount, objCount);
    }
    
    private static String[] astrCards = new String[] { "ac", "5d", "qh", "ks", "10c", "7d", "6h" };
}